/******************************************************************************
 Libreria per l'utilizzo dei ponti H realizzati da Mauro Soligo con 18F4431
 Autore:        Mauro Soligo ( tratto dal lavoro di Fabio Drago per i 18F4431)
 Versione:      1.0
 Data:          15 Novembre 2007

 Note:
 Pic:            18f4431
 H-Bridge:    RoboPonte (HIP4081A)
 MiniPonteH (L298)
 Controllo:      Locked Anti Phase
 Frequenza:      9.8kHz, 19.5kHz, 39.0kHz
 Risoluzione:    12bits, 11bit, 10bit
******************************************************************************/

#include <p18cxxx.h>
#include  "motors.h"

void initMotors(unsigned int period){

// Power Timer Control
	PTCON0 = 0b11110000;        // 1:16 postscaler, 1:1 prescale, free running
	PTCON1 = 0b10000000;        // PWM time base ON, count up
	PTPERH = (unsigned char) (period >> 8);
	PTPERL = (unsigned char) (period);

/*                PWMCON0
 bit 7 Unimplemented: Read as ?0?
 bit 6-4 PWMEN2:PWMEN0: PWM Module Enable bits(1)
 111 = All odd PWM I/O pins enabled for PWM output(2)
 110 = PWM1, PWM3 pins enabled for PWM output
 101 = All PWM I/O pins enabled for PWM output(2)
 100 = PWM0, PWM1, PWM2, PWM3, PWM4 and PWM5 pins enabled for PWM output
 011 = PWM0, PWM1, PWM2 and PWM3 I/O pins enabled for PWM output
 010 = PWM0 and PWM1 pins enabled for PWM output
 001 = PWM1 pin is enabled for PWM output
 000 = PWM module disabled; all PWM I/O pins are general purpose I/O

*/
// PWM Control
	PWMCON0 = 0b00110000;        	// 011 = PWM0, PWM1, PWM2 and PWM3 I/O pins enabled for PWM output
	PWMCON1 = 0b00000000;        	// 1:16 postscaler, updates abilitati, override asincrono

	DTCON = 0b00100000;        	// Quarzo=10Mhz; prescaler=Fosc/2 dead time 3.2uSec... 10000 = 32

// Output Override
	OVDCOND = 0b00111111;		// L'output  controllato dal DutyCycle e TimeBase
	OVDCONS = 0b00111111;		// Tutti i pin sono INACTIVE quando in override

// Fault Configuration
	FLTCONFIG = 0x00;			// Fault A e B disabilitati
	
	return ;
}

unsigned char setMotor(unsigned char motor, unsigned char enable,  int dc){
	unsigned int period;
	int dutycycle;                // nel range 0 - period
// Controlla se i parametri sono corretti
	if ((motor < 0 ) || (motor > 3)) return 1;
	if ((dc > (PWM_LIMIT)) || (dc < -(PWM_LIMIT))) return 2;
//if ((dc > 2048) || (dc < -2048)) return 2;
//Gestisce il segnale Enable da inviare al driver di potenza.

 	if(enable){
	 	switch ((unsigned char) motor){
		 	case 1 :        EN_MOT1 = ENABLE;
 							break;
 			case 2 :        EN_MOT2 = ENABLE;
 							break;
			case 3 :        EN_MOT3 = ENABLE;
 							break;
 		}
	}else{
		switch ((unsigned char) motor){
			case 1 :        EN_MOT1 = DISABLE;
							break;
 			case 2 :        EN_MOT2 = DISABLE;
 							break;
			case 3 :        EN_MOT3 = DISABLE;
							break;
 		}
 	}

 // Inverte il senso di rotazione del terzo canale
	if (motor == 3) dc = -1*dc;

 // legge la frequenza di lavoro per calcolare la risoluzione
	period = ((unsigned int)(PTPERH) << 8) + PTPERL;

 // calcola il dutycycle da applicare in base alla precisione
 	switch ((unsigned int) period){
	 	case 0x3ff : 		dutycycle = 2048 +  dc; 
	 						break;                  // 12 bit (4096)
 		case 0x1ff : 		dutycycle = 1024 + (dc / 2); 
 							break;        			// 11 bit (2048)
 		case 0x0ff : 		dutycycle = 512  + (dc / 4); 
 							break;        			// 10 bit (1024)
	}

 // Aggiorna i registri relativi al motore interessato
 	switch (motor){
	 	case 1 : 			PDC0H = dutycycle >> 8 ; 
	 						PDC0L = dutycycle; 
	 						break;
 		case 2 : 			PDC1H = dutycycle >> 8 ; 
 							PDC1L = dutycycle; 
 							break;
 		case 3 : 			PDC2H = dutycycle >> 8 ; 
 							PDC2L = dutycycle; 
 							break;
  	}
	return 0;
}
